@@ -18,18 +18,17 @@ INSERT INTO dem.message_inbox (
 ) VALUES (
 	(select pk from dem.staff where db_user = 'any-doc'),
 	(select pk_type from dem.v_inbox_item_type where type = 'memo' and category = 'administrative'),
-	'Release Notes for GNUmed 0.9.7 (database v15.7)',
-	'GNUmed 0.9.7 Release Notes:
+	'Release Notes for GNUmed 0.9.11 (database v15.11)',
+	'GNUmed 0.9.11 Release Notes:
 
-	0.9.7
+	0.9.11
 
-FIX: exception on creating allergy entry from non-brand substance intake [thanks J.Busser]
-FIX: exception on creating measurement type without LOINC [thanks J.Busser]
-FIX: exception with displaying list item tooltips [thanks Marc]
-FIX: faulty pt_BR translation ("issue name" -> "issue_name") [thanks Rogerio]
-FIX: fix DOB to be dated back one day sometimes [thanks S.Reus]
+IMPROVED: add SQL reports to find duplicate streets/urbs/regions
 
-IMPROVED: better protect against translation errors
+	15.11
+
+FIX: trf_do_not_substance_if_taken_by_patient() on ref.consumable_substance
+FIX: missing E in front of escape-syntax strings in SQL bootstrap files
 ');
 
 -- --------------------------------------------------------------
